/*
 *	PIC 12C5xx DISASSEMBLER FOR MODCHIP Ver0.11
 *	COPYRIGHT(C) 1999 Hanimar
 *	This program can be freely distributed.
 *
 *  Revision History:
 *
 *	1999-04-21 Ver0.0001
 *	1999-04-24 Ver0.1	supports full instruction set.
 *	1999-11-18 Ver0.11	fixed a bug that modisasm didn't recognize 'decf'=(
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "pic12c5x.h"


static int ReadHexCodeFrom(PIC12C5XX *pMod, FILE *fp);
static void read1line(PIC12C5XX *pMod, char *str, int lineno);

/* UTIL */
static int xtoc(const char c);
static int xstrtoi(const char *str , int bytes);

void UsageOut(const char *me)
{
	fprintf(stderr , "%s(%s)\n", TARGET, __DATE__);
	fputs("Programmed by Hanimar.\n" , stderr);
	fprintf(stderr , "USAGE: %s <hex file>\n" , me);
	exit(EXIT_FAILURE);
}

int main(int argc , char **argv)
{
	FILE *fp;
	PIC12C5XX modchip;
	/*******
	int i;
	char buf[256];
	********/

	if (argc == 1) UsageOut(argv[0]);
	if (!strcmp(argv[1] , "/?") || !strcmp(argv[1] , "-help"))
		UsageOut(argv[0]);


	PicFlash(&modchip , argv[1], C509);
	fp = fopen(argv[1] , "rt");
	if (fp == NULL) {
		fprintf(stderr , "Cannot open file [%s]\n" , argv[1]);
		exit(EXIT_FAILURE);
	}
	ReadHexCodeFrom(&modchip, fp);
	fclose(fp);

	PicPrintMore(&modchip, stdout);
	/**********
	for (i = 0; i < C509_ROMSIZE ; i ++) {
		if (PicPrint(buf, &modchip, i) > 0) fputs(buf, stdout);
	}
	**********/

	return EXIT_SUCCESS;
}

static int ReadHexCodeFrom(PIC12C5XX *pMod, FILE *fp)
{
	char buf[512];
	char *ptr;
	int lineno = 0;

	while (fgets(buf , sizeof (buf) , fp) != NULL) {
		lineno ++;
		for (ptr = buf ; *ptr == ' ' || *ptr == '\t' ; ptr ++);
		if (*ptr != ':') continue;
		read1line(pMod, ptr + 1, lineno);
	}
	return 0;
}
static void read1line(PIC12C5XX *pMod, char *str, int lineno)
{
	int sz;
	int addr;
	int i;
	int upper,lower;
	int rc;

	sz = xstrtoi(str , 1);
	sz >>= 1;
	addr = xstrtoi(&str[2] , 2);
	if (addr == 0x1ffe) {
		/* _VCLMP_OFF: 0x0fdf */
		/* _MCLRE_OFF: 0x0fef */
		/* _CP_ON    : 0x0ff7 */
		/* _WDT_OFF  : 0x0ffb */
		/* _LP_OSC   : 0x0ffc */
		/* _XT_OSC   : 0x0ffd */
		/* _INT_OSC  : 0x0ffe */
		str = &str[8];
		lower = xstrtoi(str , 1);
		upper = xstrtoi(&str[2] , 1);
		pMod->config = (int)lower + (upper << 8);
		return;
	}
	if (addr >= 0xff00) {
		/* ??? */
		return;
	}
	addr >>= 1;

	str = &str[8];
	for (i = 0 ; i < sz ; i ++) {
		lower = xstrtoi(&str[i * 4] , 1);
		upper = xstrtoi(&str[i * 4 + 2] , 1);
		rc = PicWrite(pMod, addr + i, (int)lower + (upper << 8));
		if (rc != 0) fprintf(stderr, "%s\n", PicGetLastError());
	}
	return;
}

static int xtoc(const char c)
{
	if (c >= '0' && c <= '9') return c - '0';
	if (c >= 'a' && c <= 'f') return 10 + c - 'a';
	if (c >= 'A' && c <= 'F') return 10 + c - 'A';
	return -1;
}

/*
 *	bytes must be less than 5. 
 */
static int xstrtoi(const char *str , int bytes)
{
	int i;
	int c;
	int ret = 0;

	for (i = 0 ; i < bytes ; i ++) {
		c = (xtoc(str[i*2]) << 4) + xtoc(str[i*2+1]);
		ret |= c << (8 * (bytes-1-i));
	}
	return ret;
}
