/*
 *	PIC 12C5xx DISASSEMBLER FOR MODCHIP Ver0.11
 *	COPYRIGHT(C) 1999 Hanimar
 *	This program can be freely distributed.
 */
#ifndef _PIC12C5XX_H_
#define _PIC12C5XX_H_
#include <stdio.h>
#define TARGET	"PIC 12C5xx DISASSEMBLER FOR MODCHIP Ver0.11"

#define C509_ROMSIZE	1024	/* x 12bits */
//#define C509_RAMSIZE	41

#define C508_ROMSIZE	512		/* x 12bits */
//#define C508_RAMSIZE	25

/*
 *	RAM
 *	12C508 07-1f
 *	12C509 07-1f(Bank0),30-3f(Bank1)
 */
#define C509_RAMSIZE	0x40

/*
 *	Configuration Bits
 */
#define _VCLMP_ON	0x0FFF
#define _VCLMP_OFF	0x0FDF
#define _MCLRE_ON	0x0FFF
#define _MCLRE_OFF	0x0FEF
#define _CP_ON		0x0FF7
#define _CP_OFF		0x0FFF
#define _WDT_ON		0x0FFF
#define _WDT_OFF	0x0FFB
#define _LP_OSC		0x0FFC
#define _XT_OSC		0x0FFD
#define _INT_OSC	0x0FFE
#define _RC_OSC		0x0FFF

/*
 *	Register
 */
#define REG_INDF	0
#define REG_TMR0	1
#define REG_PCL		2
#define REG_STATUS	3
#define REG_FSR		4
#define REG_OSCCAL	5
#define REG_GPIO	6
#define REG_MAX		6
/*
 *	MNEMONIC
 */
#define MN_UNKNOWN	0

#define MN_ADDWF	1
#define MN_ANDWF	2
#define MN_CLRF		3
#define MN_CLRW		4
#define MN_COMF		5
#define MN_DECF		6
#define MN_DECFSZ	7
#define MN_INCF		8
#define MN_INCFSZ	9
#define MN_IORWF	10
#define MN_MOVF		11
#define MN_MOVWF	12
#define MN_NOP		13
#define MN_RLF		14
#define MN_RRF		15
#define MN_SUBWF	16
#define MN_SWAPF	17
#define MN_XORWF	18

#define MN_BCF		19
#define MN_BSF		20
#define MN_BTFSC	21
#define MN_BTFSS	22

#define MN_ANDLW	23
#define MN_CALL		24
#define MN_CLRWDT	25
#define MN_GOTO		26
#define MN_IORLW	27
#define MN_MOVLW	28
#define MN_OPTION	29
#define MN_RETLW	30
#define MN_SLEEP	31
#define MN_TRIS		32
#define MN_XORLW	33

#define MN_LAST		33

typedef struct {
	int code;
	int flags;
#define FLAG_USED	(1 << 0)
#define FLAG_CALLED	(1 << 1)
#define FLAG_JUMPED	(1 << 2)
#define FLAG_BRANCH	(1 << 3)

	char mnemonic;
	int op1,op2;
#define OPERAND_UNUSED	-1
} OPCODE;

typedef enum {
	C508 , C509
} ROMTYPE;

typedef struct {
	char fname[128];
	OPCODE rom[C509_ROMSIZE];
	unsigned char ram[C509_RAMSIZE];
#define RAM_USED	1

	int gpioFlag;
	int config;
	ROMTYPE romType;

} PIC12C5XX;

#ifdef __cplusplus
extern "C" {
#endif

const char *PicGetLastError(void);
void PicFlash(PIC12C5XX *pMod, const char *fname, ROMTYPE romType);
int PicWrite(PIC12C5XX *pMod, int address, int opcode);
int PicPrint(char *dest, PIC12C5XX *pMod, int address);

int PicPrintMore(PIC12C5XX *pMod, FILE *fpout);

#ifdef __cplusplus
}
#endif

#ifdef IAMTHEOWNER
#define extern
#endif
extern char *Mnemonics[];

#ifdef IAMTHEOWNER
#undef extern
#endif

#endif /* _PIC12C5XX_H_ */
